<?xml version='1.0' encoding='utf-8'?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:r='http://relaxng.org/ns/structure/1.0'
                xmlns:a='http://bulletin.cstug.cz/ns/annotation'
                xmlns:saxon='http://saxon.sf.net/'
                xmlns:xs='http://www.w3.org/2001/XMLSchema'
                exclude-result-prefixes='a r xs'>

<!--
Stylesheet to convert RNG schemas to HTML.
All elements with namespace URI http://bulletin.cstug.cz/ns/annotation, which are not explicitely
handled, will be converted to the HTML elements with the same local name and with copies of all
attributes.
-->

<xsl:strip-space elements='r:grammar'/>
<xsl:preserve-space elements='r:define r:start'/>

<xsl:param name='maxcol' as='xs:integer' select='70'/>

<xsl:output method='html' indent='yes'/>

<xsl:variable name='basename'>
  <xsl:if test='function-available("saxon:system-id")'>
    <xsl:analyze-string select='saxon:system-id()' regex='[^/]+$'>
      <xsl:matching-substring>
        <xsl:value-of select='.'/>
      </xsl:matching-substring>
    </xsl:analyze-string>
  </xsl:if>
</xsl:variable>

<xsl:template match='r:grammar'>
  <html>
    <head>
      <title><xsl:value-of select='a:title'/></title>
    </head>
    <body background="back_narrow.gif"
          bgcolor="#ffffff"
          text="#000000"
          alink="#c14242"
          vlink="#d30000"
          link="#900000" >
<table border='0' width='100%'>
 <tr valign='top'>
  <td width='30'>&#160;</td>
  <td>
      <xsl:apply-templates/>
</td></tr></table>
    </body>
  </html>
</xsl:template>

<xsl:template match='a:cstugns'>
  <xsl:value-of select='/r:grammar/@ns'/>
</xsl:template>

<xsl:template match='a:title'>
   <a href='http://bulletin.cstug.cz/xml/'>
     <img src='logo.gif' border='0' height='51' width='165' alt='CSTUG' align='right'/>
   </a>
  <h1><xsl:apply-templates/></h1>
  <xsl:if test='$basename != ""'>
    <p><a title='{$basename}' href='{$basename}'>
      <xsl:value-of select='$basename'/>
    </a></p>
  </xsl:if>
  <h2><xsl:text>Table of contents</xsl:text></h2>
  <ul>
    <xsl:for-each select='../r:define|../r:start'>
      <li>
        <xsl:choose>
          <xsl:when test='name(.)="define"'>
            <a title='{@name}'>
              <xsl:attribute name='href'>
                <xsl:value-of select='concat("#", @name)'/>
              </xsl:attribute>
              <xsl:text>&lt;define name="</xsl:text>
              <xsl:value-of select='@name'/>
              <xsl:text>"&gt;</xsl:text>
            </a>
          </xsl:when>
          <xsl:when test='name(.)="start"'>
            <a title='start' href='#start'><xsl:text>&lt;start&gt;</xsl:text></a>
          </xsl:when>
        </xsl:choose>
      </li>
    </xsl:for-each>
  </ul>
</xsl:template>

<xsl:template match='a:annotation'>
  <xsl:choose>
    <xsl:when test='name(following-sibling::element()[1]) = "define"'>
      <h2><a>
        <xsl:attribute name='name'>
          <xsl:value-of select='following-sibling::element()[1]/@name'/>
        </xsl:attribute>
        <xsl:text>&lt;define name="</xsl:text>
        <xsl:value-of select='following-sibling::element()[1]/@name'/>
        <xsl:text>"&gt;</xsl:text>
      </a></h2>
    </xsl:when>
    <xsl:when test='name(following-sibling::element()[1]) = "start"'>
      <h2><a name="start">
        <xsl:text>&lt;start&gt;</xsl:text>
      </a></h2>
    </xsl:when>
  </xsl:choose>
  <xsl:choose>
    <xsl:when test='a:p'>
      <xsl:apply-templates/>
    </xsl:when>
    <xsl:otherwise>
      <p>
        <xsl:apply-templates/>
      </p>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match='a:elem'>
  <code>&lt;<xsl:apply-templates/>&gt;</code>
</xsl:template>

<xsl:template match='r:define'>
<xsl:variable name='name'><xsl:value-of select='@name'/></xsl:variable>
<p><b>The pattern is used in:</b></p>
<ul>
  <xsl:for-each select='/r:grammar/r:define'>
    <xsl:if test='.//r:ref[@name=$name]'>
      <li>
        <a title='{@name}'>
          <xsl:attribute name='href'>
            <xsl:value-of select='concat("#", @name)'/>
          </xsl:attribute>
          <xsl:text>&lt;define name="</xsl:text>
          <xsl:value-of select='@name'/>
          <xsl:text>"&gt;</xsl:text>
        </a>
      </li>
    </xsl:if>
  </xsl:for-each>
  <xsl:if test='/r:grammar/r:start//r:ref[@name=$name]'>
    <li><a title='start' href='#start'>&lt;start&gt;</a></li>
  </xsl:if>
</ul>
<pre>
  <xsl:text>&lt;define name="</xsl:text><xsl:value-of select='@name'/><xsl:text>"&gt;</xsl:text>
  <xsl:apply-templates/>
  <xsl:text>&lt;/define&gt;</xsl:text>
</pre>
</xsl:template>

<xsl:template match='r:start'>
<pre>
  <xsl:text>&lt;start&gt;</xsl:text>
  <xsl:apply-templates/>
  <xsl:text>&lt;/start&gt;</xsl:text>
</pre>
</xsl:template>

<xsl:template match='*'>
  <xsl:choose>
    <xsl:when test='namespace-uri(.)="http://bulletin.cstug.cz/ns/annotation"'>
      <xsl:element name='{local-name(.)}'>
        <xsl:apply-templates select='@*' mode='doc'/>
        <xsl:apply-templates/>
      </xsl:element>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>&lt;</xsl:text><xsl:value-of select='name(.)'/>
      <xsl:apply-templates select='@*'/>
      <xsl:choose>
        <xsl:when test='count(*)=0 and string-length(normalize-space(.))=0'>
          <xsl:text>/&gt;</xsl:text>
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>&gt;</xsl:text>
          <xsl:apply-templates/>
          <xsl:text>&lt;/</xsl:text><xsl:value-of select='name(.)'/><xsl:text>&gt;</xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match='@*' mode='doc'>
  <xsl:copy/>
</xsl:template>

<xsl:template match='@*'>
  <xsl:text> </xsl:text>
  <xsl:value-of select='name(.)'/>
  <xsl:text>="</xsl:text>
  <xsl:choose>
    <xsl:when test='name(..) = "ref"'>
      <a title='{.}'>
        <xsl:attribute name='href'>
          <xsl:value-of select='concat("#", .)'/>
        </xsl:attribute>
        <xsl:value-of select='.'/>
      </a>
    </xsl:when>
    <xsl:otherwise>
      <xsl:choose>
        <xsl:when test='string-length(.) &lt;= $maxcol'>
          <xsl:value-of select='.'/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:analyze-string select='.' regex='(.{{1,{$maxcol}}})\s+'>
            <xsl:matching-substring>
              <xsl:value-of select='concat(regex-group(1), "&#10;")'/>
            </xsl:matching-substring>
            <xsl:non-matching-substring>
              <xsl:value-of select='.'/>
            </xsl:non-matching-substring>
          </xsl:analyze-string>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:otherwise>
  </xsl:choose>
  <xsl:text>"</xsl:text>
</xsl:template>

</xsl:stylesheet>
